package com.ruoyi.web.runner;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.TypeReference;
import com.ruoyi.framework.datasource.DynamicDataSource;
import com.ruoyi.quartz.glue.GlueFactory;
import com.ruoyi.quartz.handler.impl.MyJobHandler;
import com.ruoyi.quartz.task.ChangedSave;
import com.ruoyi.ucp.entity.Formula;
import com.ruoyi.ucp.entity.Method;
import com.ruoyi.ucp.entity.ProEconWtCurveFittingMonth;
import com.ruoyi.ucp.entity.UcpDataSource;
import com.ruoyi.ucp.feign.AdapterApi;
import com.ruoyi.quartz.handler.IJobHandler;
import com.ruoyi.ucp.service.*;
import com.ruoyi.ucp.util.CalcCache;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

@Component
public class LoadDataSourceRunner implements CommandLineRunner {
    @Resource
    private DynamicDataSource dynamicDataSource;
    @Resource
    private IUcpDataSourceService ucpDataSourceService;

    //@Resource
    //private ScriptEngine engine;
    @Resource
    private AdapterApi adapter;
    @Resource
    private IPointInfoService pointService;
    @Resource
    private IStationInfoDayService stationInfoDayService;
    @Resource
    private IStationInfoHourService stationInfoHourService;
    @Resource
    private IMethodService methodService;
    @Resource
    private ILineInfoDayService lineInfoDayService;
    @Resource
    private IFormulaService formulaService;
    @Resource
    @Autowired
    private RedisTemplate redisTemplate;

    @Override
    public void run(String... args) {
        List<UcpDataSource> list = ucpDataSourceService.list();
        List<DruidDataSource> dataSources = BeanUtil.copyToList(list, DruidDataSource.class);
        dynamicDataSource.createDataSource(dataSources);

        //engine.put("adapter", adapter);
        //engine.put("pointService", pointService);
        //engine.put("stationInfoDayService", stationInfoDayService);
        //engine.put("stationInfoHourService", stationInfoHourService);
        //engine.put("lineInfoDayService", lineInfoDayService);

        List<Method> methods = methodService.list();
        List<Formula> formulas = formulaService.list();
        String functionCode = GlueFactory.getFunctionTemplate();
        try {
            for (Method method : methods) {
                //engine.eval(method.getMethodExpression());
                functionCode = functionCode
                        .replace("//此处加公式", method.getMethodExpression() + "\n//此处加公式");
            }
            for (Formula formula : formulas) {
                functionCode = functionCode.replace("//此处加公式", formula.getFormulaExpression() + "\n//此处加公式");
            }
            IJobHandler jobHandler = GlueFactory.getInstance().loadNewInstance(functionCode);
            MyJobHandler.jobHandler = jobHandler;
        } catch (Exception e) {
            e.printStackTrace();
        }

        LettuceConnectionFactory factory = (LettuceConnectionFactory) redisTemplate.getConnectionFactory();
        factory.setDatabase(9);
        redisTemplate.setConnectionFactory(factory);
        Set<String> keys = redisTemplate.keys("monthCurve_");
        for (String key : keys) {
            //redis缓存中获取到字符串
            String o = (String) redisTemplate.opsForValue().get(key);
            //字符串类型转换为list月拟合曲线集合
            List<ProEconWtCurveFittingMonth> powerList = JSONObject.parseObject(o, new TypeReference<List<ProEconWtCurveFittingMonth>>() {});
            Map<Double, Double> collect = powerList.stream().collect(Collectors.toMap(ProEconWtCurveFittingMonth::getSpeed, ProEconWtCurveFittingMonth::getActualPower));
            CalcCache.wtMcfMap.put(powerList.get(0).getWindturbineId(), collect);
        }
        factory.setDatabase(5);
    }
}
